<?php

namespace Drupal\nft_field\Plugin\Field\FieldType;

use Drupal\Core\Field\FieldItemBase;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\TypedData\DataDefinition;
use Drupal\nft_field\NftItemInterface;

/**
 * Plugin implementation of the 'nft' field type.
 *
 * @FieldType(
 *   id = "nft",
 *   label = @Translation("Nft"),
 *   description = @Translation("Stores a contract address and a token ID to assemble an NFT."),
 *   default_widget = "nft_default",
 *   default_formatter = "string",
 *   constraints = {}
 * )
 */
class NftItem extends FieldItemBase implements NftItemInterface {

  /**
   * {@inheritdoc}
   */
  public static function propertyDefinitions(FieldStorageDefinitionInterface $field_definition) {
    $properties['contract_address'] = DataDefinition::create('string')
    // @todo Create custom constraint to validate contract.
    // ->setConstraints([
    //    'ContractAddress' => []
    // ])
      ->setRequired(TRUE)
      ->setLabel(t('Contract address'));

    $properties['token_id'] = DataDefinition::create('string')
      ->setRequired(TRUE)
      ->setLabel(t('Token ID'));

    $properties['network_id'] = DataDefinition::create('string')
      ->setRequired(TRUE)
      ->setLabel(t('Network ID'));

    return $properties;
  }

  /**
   * {@inheritdoc}
   */
  public static function schema(FieldStorageDefinitionInterface $field_definition) {
    return [
      'columns' => [
        'contract_address' => [
          'description' => 'The address of the contract.',
          'type' => 'varchar',
          'length' => 255,
        ],
        'token_id' => [
          'description' => 'The token ID.',
          'type' => 'varchar',
          'length' => 255,
        ],
        'network_id' => [
          'description' => 'The network ID.',
          'type' => 'varchar',
          'length' => 255,
        ],
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public static function mainPropertyName() {
    // No need to have a default property.
    // Default value is built in source plugin.
    // @see \Drupal\nft_media\Plugin\media\Source\Nft::getSourceFieldValue();
    return NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function isEmpty() {
    $value = $this->get('contract_address')->getValue();
    $value .= $this->get('token_id')->getValue();
    $value .= $this->get('network_id')->getValue();
    return $value === NULL || $value === '';
  }
}
