<?php

namespace Drupal\nft_field\Plugin\Field\FieldWidget;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\nft_field\NftItemInterface;

/**
 * Plugin implementation of the 'nft' widget.
 *
 * @FieldWidget(
 *   id = "nft_default",
 *   label = @Translation("NFT"),
 *   field_types = {
 *     "nft"
 *   }
 * )
 */
class NftWidget extends WidgetBase {

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    /** @var \Drupal\nft\NftItemInterface $item */
    $item = $items[$delta];

    $element['contract_address'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Contract address'),
      '#default_value' => $items[$delta]->contract_address,
      '#element_validate' => [[static::class, 'validateContractElement']],
      '#maxlength' => 2048,
      '#required' => TRUE,
    ];

    $element['token_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Token ID'),
      '#default_value' => $items[$delta]->token_id ?? NULL,
      '#min' => 0,
      '#required' => TRUE,
    ];

    $element['network_id'] = [
      '#type' => 'number',
      '#title' => $this->t('Network ID'),
      '#default_value' => $items[$delta]->network_id ?? 1,
      '#min' => 1,
      '#required' => TRUE,
    ];

    return $element;
  }

  /**
   * Form element validation handler for the 'contract' element.
   *
   * @todo Add proper contract address format validation.
   */
  public static function validateContractElement($element, FormStateInterface $form_state, $form) {
    $address = $element['#value'];
    if (strpos($address, '0x') !== 0) {
      $form_state->setError($element, t('Contract address must start with "0x".'));
      return FALSE;
    }
  }

}
